#define G_LOG_DOMAIN "gpod-downloader"

#include "config.h"

#include "gpod-downloader.h"

#include <errno.h>
#include <libsoup/soup.h>
#include <glib.h>
#include <glib-object.h>

/**
 * SECTION:gpod-downloader
 * @title: GpodDownloader
 * @short_description: A resource downloader
 * @include: gpod-downloader.h
 *
 * Downloads resource from the web and caches them.
 **/

typedef struct _GpodDownloaderPrivate
{
    gchar *directory;
} GpodDownloaderPrivate;

struct _GpodDownloader
{
    GObject parent_instance;
};

G_DEFINE_TYPE_WITH_PRIVATE (GpodDownloader, gpod_downloader, G_TYPE_OBJECT)

enum {
    PROP_0,
    PROP_DIRECTORY,
    LAST_PROP
};

static GParamSpec *properties[LAST_PROP] = { NULL, };

enum
{
    DOWNLOADING,
    DOWNLOADED,
    NUM_SIGNALS
};

static guint signals[NUM_SIGNALS] = { 0, };

static void
create_dir (gchar *directory)
{
    if (g_mkdir_with_parents (directory, 0777) != 0 &&
        g_file_error_from_errno (errno) != G_FILE_ERROR_EXIST)
        g_warning ("Error creating directory at: \"%s\"", directory);

    g_debug ("Directory created at: \"%s\"", directory);

    g_free (directory);
}

static gchar *
get_hash (gchar *string)
{
    GChecksum *checksum = g_checksum_new (G_CHECKSUM_MD5);
    gchar *hash;

    g_checksum_update (checksum, (guchar *) string, g_strv_length (&string));
    hash = g_strdup (g_checksum_get_string (checksum));
    g_checksum_free (checksum);

    return hash;
}

static void
save_file_to_dir (GpodDownloader *self,
                  gchar          *directory,
                  gchar          *string)
{
    g_return_if_fail (GPOD_IS_DOWNLOADER (self));

    //gchar *hash;

    //hash = get_hash (string);

    //g_free (hash);
}


static void
save_file (GpodDownloader *self,
           gchar          *string)
{

}

static void
gpod_downloader_finalize (GObject *object)
{
    GpodDownloader *self = GPOD_DOWNLOADER (object);
    GpodDownloaderPrivate *priv = gpod_downloader_get_instance_private (self);

    if (priv->directory)
        g_free (priv->directory);

    G_OBJECT_CLASS (gpod_downloader_parent_class)->finalize (object);
}

static void
gpod_downloader_set_property (GObject      *object,
                              guint         property_id,
                              const GValue *value,
                              GParamSpec   *pspec)
{
    GpodDownloader *self = GPOD_DOWNLOADER (object);
    GpodDownloaderPrivate *priv = gpod_downloader_get_instance_private (self);

    switch (property_id)
    {
        case PROP_DIRECTORY:
            priv->directory = g_value_dup_string (value);
            create_dir (priv->directory);
            break;

        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
            break;
    }
}

static void
gpod_downloader_get_property (GObject    *object,
                              guint       property_id,
                              GValue     *value,
                              GParamSpec *pspec)
{
    GpodDownloader *self = GPOD_DOWNLOADER (object);
    GpodDownloaderPrivate *priv = gpod_downloader_get_instance_private (self);

    switch (property_id)
    {
        case PROP_DIRECTORY:
            g_value_set_string (value, priv->directory);
            break;

        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
            break;
    }
}

static void
gpod_downloader_class_init (GpodDownloaderClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    object_class->finalize = gpod_downloader_finalize;
    object_class->set_property = gpod_downloader_set_property;
    object_class->get_property = gpod_downloader_get_property;

    /**
    * GpodWindow::directoryu:
    *
    * The base directory to save files into.
    */
    properties[PROP_DIRECTORY] =
        g_param_spec_object ("directory",
                             "Directory",
                             "The base directory to save files into.",
                             G_TYPE_FROM_CLASS (object_class),
                             G_PARAM_READWRITE);

    g_object_class_install_properties (object_class,
                                       LAST_PROP,
                                       properties);

    signals[DOWNLOADING] =
        g_signal_new ("loading",
                      G_TYPE_FROM_CLASS (object_class),
                      G_SIGNAL_RUN_LAST,
                      0,
                      NULL,
                      NULL,
                      NULL,
                      G_TYPE_NONE,
                      1,
                      G_TYPE_STRING);

    signals[DOWNLOADED] =
        g_signal_new ("loading-finished",
                      G_TYPE_FROM_CLASS (object_class),
                      G_SIGNAL_RUN_LAST,
                      0,
                      NULL,
                      NULL,
                      NULL,
                      G_TYPE_NONE,
                      1,
                      G_TYPE_STRING);
}

static void
gpod_downloader_init (GpodDownloader *self)
{
    g_object_set (self,
                  "directory", g_build_filename (g_get_user_cache_dir (), PACKAGE, "resources", NULL),
                  NULL);
}

GpodDownloader *
gpod_downloader_new (void)
{
    return g_object_new (GPOD_TYPE_DOWNLOADER, NULL);
}

void
gpod_downloader_save_image (GpodDownloader *self,
                            gchar          *file)
{
    GpodDownloaderPrivate *priv = gpod_downloader_get_instance_private (self);

    create_dir (g_build_filename (priv->directory, "images", NULL));

    save_file (self, file);
}

void
gpod_downloader_save_audio (GpodDownloader *self,
                            gchar          *file)
{
    save_file (self, file);
}

void
gpod_downloader_save (GpodDownloader *self,
                      gchar          *file)
{

}
